@@ -8,36 +8,27 @@
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA 
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #ifndef __SBSERVICES_H
 #define __SBSERVICES_H
 
-#ifdef WIN32
-#include <windows.h>
-#else
-#include <pthread.h>
-#endif
-
 #include "libimobiledevice/sbservices.h"
 #include "property_list_service.h"
+#include "common/thread.h"
 
 struct sbservices_client_private {
 	property_list_service_client_t parent;
-#ifdef WIN32
-	CRITICAL_SECTION mutex;
-#else
-	pthread_mutex_t mutex;
-#endif
+	mutex_t mutex;
 };
 
 #endif
