@@ -9,15 +9,15 @@
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA 
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -43,6 +43,7 @@
 #include <libimobiledevice/mobilebackup.h>
 #include <libimobiledevice/notification_proxy.h>
 #include <libimobiledevice/afc.h>
+#include "common/utils.h"
 
 #define MOBILEBACKUP_SERVICE_NAME "com.apple.mobilebackup"
 #define NP_SERVICE_NAME "com.apple.mobile.notification_proxy"
@@ -51,12 +52,13 @@
 #define LOCK_WAIT 200000
 
 #ifdef WIN32
+#include <windows.h>
 #define sleep(x) Sleep(x*1000)
 #endif
 
 static mobilebackup_client_t mobilebackup = NULL;
 static lockdownd_client_t client = NULL;
-static idevice_t phone = NULL;
+static idevice_t device = NULL;
 
 static int quit_flag = 0;
 
@@ -66,11 +68,6 @@ enum cmd_mode {
 	CMD_LEAVE
 };
 
-enum plist_format_t {
-	PLIST_FORMAT_XML,
-	PLIST_FORMAT_BINARY
-};
-
 enum device_link_file_status_t {
 	DEVICE_LINK_FILE_STATUS_NONE = 0,
 	DEVICE_LINK_FILE_STATUS_HUNK,
@@ -227,68 +224,11 @@ static void notify_cb(const char *notification, void *userdata)
 	}
 }
 
-static char *str_toupper(char* str)
-{
-	char *res = strdup(str);
-	unsigned int i;
-	for (i = 0; i < strlen(res); i++) {
-		res[i] = toupper(res[i]);
-	}
-	return res;
-}
-
-static char* build_path(const char* elem, ...)
-{
-	if (!elem) return NULL;
-	va_list args;
-	int len = strlen(elem)+1;
-	va_start(args, elem);
-	char *arg = va_arg(args, char*);
-	while (arg) {
-		len += strlen(arg)+1;
-		arg = va_arg(args, char*);
-	}
-	va_end(args);
-
-	char* out = (char*)malloc(len);
-	strcpy(out, elem);
-
-	va_start(args, elem);
-	arg = va_arg(args, char*);
-	while (arg) {
-		strcat(out, "/");
-		strcat(out, arg);
-		arg = va_arg(args, char*);
-	}
-	va_end(args);
-	return out;
-}
-
-static char* format_size_for_display(uint64_t size)
-{
-	char buf[32];
-	double sz;
-	if (size >= 1000000000LL) {
-		sz = ((double)size / 1000000000.0f);
-		sprintf(buf, "%0.1f GB", sz);
-	} else if (size >= 1000000LL) {
-		sz = ((double)size / 1000000.0f);
-		sprintf(buf, "%0.1f MB", sz);
-	} else if (size >= 1000LL) {
-		sz = ((double)size / 1000.0f);
-		sprintf(buf, "%0.1f kB", sz);
-	} else {
-		sprintf(buf, "%d Bytes", (int)size);
-	}
-	return strdup(buf);
-}
-
-static plist_t mobilebackup_factory_info_plist_new()
+static plist_t mobilebackup_factory_info_plist_new(const char* udid)
 {
 	/* gather data from lockdown */
 	plist_t value_node = NULL;
 	plist_t root_node = NULL;
-	char *udid = NULL;
 	char *udid_uppercase = NULL;
 
 	plist_t ret = plist_new_dict();
@@ -298,44 +238,42 @@ static plist_t mobilebackup_factory_info_plist_new()
 
 	/* set fields we understand */
 	value_node = plist_dict_get_item(root_node, "BuildVersion");
-	plist_dict_insert_item(ret, "Build Version", plist_copy(value_node));
+	plist_dict_set_item(ret, "Build Version", plist_copy(value_node));
 
 	value_node = plist_dict_get_item(root_node, "DeviceName");
-	plist_dict_insert_item(ret, "Device Name", plist_copy(value_node));
-	plist_dict_insert_item(ret, "Display Name", plist_copy(value_node));
+	plist_dict_set_item(ret, "Device Name", plist_copy(value_node));
+	plist_dict_set_item(ret, "Display Name", plist_copy(value_node));
 
 	/* FIXME: How is the GUID generated? */
-	plist_dict_insert_item(ret, "GUID", plist_new_string("---"));
+	plist_dict_set_item(ret, "GUID", plist_new_string("---"));
 
 	value_node = plist_dict_get_item(root_node, "InternationalMobileEquipmentIdentity");
 	if (value_node)
-		plist_dict_insert_item(ret, "IMEI", plist_copy(value_node));
+		plist_dict_set_item(ret, "IMEI", plist_copy(value_node));
 
-	plist_dict_insert_item(ret, "Last Backup Date", plist_new_date(time(NULL), 0));
+	plist_dict_set_item(ret, "Last Backup Date", plist_new_date(time(NULL), 0));
 
 	value_node = plist_dict_get_item(root_node, "ProductType");
-	plist_dict_insert_item(ret, "Product Type", plist_copy(value_node));
+	plist_dict_set_item(ret, "Product Type", plist_copy(value_node));
 
 	value_node = plist_dict_get_item(root_node, "ProductVersion");
-	plist_dict_insert_item(ret, "Product Version", plist_copy(value_node));
+	plist_dict_set_item(ret, "Product Version", plist_copy(value_node));
 
 	value_node = plist_dict_get_item(root_node, "SerialNumber");
-	plist_dict_insert_item(ret, "Serial Number", plist_copy(value_node));
+	plist_dict_set_item(ret, "Serial Number", plist_copy(value_node));
 
 	value_node = plist_dict_get_item(root_node, "UniqueDeviceID");
-	idevice_get_udid(phone, &udid);
-	plist_dict_insert_item(ret, "Target Identifier", plist_new_string(udid));
+	plist_dict_set_item(ret, "Target Identifier", plist_new_string(udid));
 
 	/* uppercase */
-	udid_uppercase = str_toupper(udid);
-	plist_dict_insert_item(ret, "Unique Identifier", plist_new_string(udid_uppercase));
+	udid_uppercase = string_toupper((char*)udid);
+	plist_dict_set_item(ret, "Unique Identifier", plist_new_string(udid_uppercase));
 	free(udid_uppercase);
-	free(udid);
 
 	/* FIXME: Embed files as <data> nodes */
 	plist_t files = plist_new_dict();
-	plist_dict_insert_item(ret, "iTunes Files", files);
-	plist_dict_insert_item(ret, "iTunes Version", plist_new_string("9.0.2"));
+	plist_dict_set_item(ret, "iTunes Files", files);
+	plist_dict_set_item(ret, "iTunes Version", plist_new_string("9.0.2"));
 
 	plist_free(root_node);
 
@@ -355,89 +293,6 @@ static void mobilebackup_info_update_last_backup_date(plist_t info_plist)
 	node = NULL;
 }
 
-static void buffer_read_from_filename(const char *filename, char **buffer, uint64_t *length)
-{
-	FILE *f;
-	uint64_t size;
-
-	*length = 0;
-
-	f = fopen(filename, "rb");
-	if (!f) {
-		return;
-	}
-
-	fseek(f, 0, SEEK_END);
-	size = ftell(f);
-	rewind(f);
-
-	if (size == 0) {
-		return;
-	}
-
-	*buffer = (char*)malloc(sizeof(char)*size);
-	fread(*buffer, sizeof(char), size, f);
-	fclose(f);
-
-	*length = size;
-}
-
-static void buffer_write_to_filename(const char *filename, const char *buffer, uint64_t length)
-{
-	FILE *f;
-
-	f = fopen(filename, "ab");
-	fwrite(buffer, sizeof(char), length, f);
-	fclose(f);
-}
-
-static int plist_read_from_filename(plist_t *plist, const char *filename)
-{
-	char *buffer = NULL;
-	uint64_t length;
-
-	if (!filename)
-		return 0;
-
-	buffer_read_from_filename(filename, &buffer, &length);
-
-	if (!buffer) {
-		return 0;
-	}
-
-	if ((length > 8) && (memcmp(buffer, "bplist00", 8) == 0)) {
-		plist_from_bin(buffer, length, plist);
-	} else {
-		plist_from_xml(buffer, length, plist);
-	}
-
-	free(buffer);
-
-	return 1;
-}
-
-static int plist_write_to_filename(plist_t plist, const char *filename, enum plist_format_t format)
-{
-	char *buffer = NULL;
-	uint32_t length;
-
-	if (!plist || !filename)
-		return 0;
-
-	if (format == PLIST_FORMAT_XML)
-		plist_to_xml(plist, &buffer, &length);
-	else if (format == PLIST_FORMAT_BINARY)
-		plist_to_bin(plist, &buffer, &length);
-	else
-		return 0;
-
-	buffer_write_to_filename(filename, buffer, length);
-
-	free(buffer);
-
-	return 1;
-}
-
 static int plist_strcmp(plist_t node, const char *str)
 {
 	char *buffer = NULL;
@@ -455,10 +310,11 @@ static int plist_strcmp(plist_t node, const char *str)
 
 static char *mobilebackup_build_path(const char *backup_directory, const char *name, const char *extension)
 {
-	char* filename = (char*)malloc(strlen(name)+strlen(extension)+1);
+	char* filename = (char*)malloc(strlen(name)+(extension == NULL ? 0: strlen(extension))+1);
 	strcpy(filename, name);
-	strcat(filename, extension);
-	char *path = build_path(backup_directory, filename, NULL);
+	if (extension != NULL)
+		strcat(filename, extension);
+	char *path = string_build_path(backup_directory, filename, NULL);
 	free(filename);
 	return path;
 }
@@ -467,7 +323,7 @@ static void mobilebackup_write_status(const char *path, int status)
 {
 	struct stat st;
 	plist_t status_plist = plist_new_dict();
-	plist_dict_insert_item(status_plist, "Backup Success", plist_new_bool(status));
+	plist_dict_set_item(status_plist, "Backup Success", plist_new_bool(status));
 	char *file_path = mobilebackup_build_path(path, "Status", ".plist");
 
 	if (stat(file_path, &st) == 0)
@@ -655,13 +511,13 @@ static int mobilebackup_check_file_integrity(const char *backup_directory, const
 
 	char *version = NULL;
 	node = plist_dict_get_item(metadata, "Version");
-	if (node && (plist_get_node_type(node) == PLIST_STRING)) { 
+	if (node && (plist_get_node_type(node) == PLIST_STRING)) {
 		plist_get_string_val(node, &version);
 	}
 
 	char *destpath = NULL;
 	node = plist_dict_get_item(metadata, "Path");
-	if (node && (plist_get_node_type(node) == PLIST_STRING)) { 
+	if (node && (plist_get_node_type(node) == PLIST_STRING)) {
 		plist_get_string_val(node, &destpath);
 	}
 
@@ -673,7 +529,7 @@ static int mobilebackup_check_file_integrity(const char *backup_directory, const
 
 	char *domain = NULL;
 	node = plist_dict_get_item(metadata, "Domain");
-	if (node && (plist_get_node_type(node) == PLIST_STRING)) { 
+	if (node && (plist_get_node_type(node) == PLIST_STRING)) {
 		plist_get_string_val(node, &domain);
 	}
 
@@ -691,7 +547,7 @@ static int mobilebackup_check_file_integrity(const char *backup_directory, const
 		snprintf (p, 3, "%02x", (unsigned char)fnhash[i] );
 	}
 	if (strcmp(fnamehash, hash)) {
-		printf("\r\n"); 
+		printf("\r\n");
 		printf("WARNING: filename hash does not match for entry '%s'\n", hash);
 	}
 
@@ -746,18 +602,18 @@ static int mobilebackup_check_file_integrity(const char *backup_directory, const
 
 static void do_post_notification(const char *notification)
 {
-	uint16_t nport = 0;
+	lockdownd_service_descriptor_t service = NULL;
 	np_client_t np;
 
 	if (!client) {
-		if (lockdownd_client_new_with_handshake(phone, &client, "idevicebackup") != LOCKDOWN_E_SUCCESS) {
+		if (lockdownd_client_new_with_handshake(device, &client, "idevicebackup") != LOCKDOWN_E_SUCCESS) {
 			return;
 		}
 	}
 
-	lockdownd_start_service(client, NP_SERVICE_NAME, &nport);
-	if (nport) {
-		np_client_new(phone, nport, &np);
+	lockdownd_start_service(client, NP_SERVICE_NAME, &service);
+	if (service && service->port) {
+		np_client_new(device, service, &np);
 		if (np) {
 			np_post_notification(np, notification);
 			np_client_free(np);
@@ -765,6 +621,11 @@ static void do_post_notification(const char *notification)
 	} else {
 		printf("Could not start %s\n", NP_SERVICE_NAME);
 	}
+
+	if (service) {
+		lockdownd_service_descriptor_free(service);
+		service = NULL;
+	}
 }
 
 static void print_progress(double progress)
@@ -813,15 +674,16 @@ static void print_usage(int argc, char **argv)
 	printf("  -u, --udid UDID\ttarget specific device by its 40-digit device UDID\n");
 	printf("  -h, --help\t\tprints usage information\n");
 	printf("\n");
+	printf("Homepage: <http://libimobiledevice.org>\n");
 }
 
 int main(int argc, char *argv[])
 {
 	idevice_error_t ret = IDEVICE_E_UNKNOWN_ERROR;
+	lockdownd_error_t ldret = LOCKDOWN_E_UNKNOWN_ERROR;
 	int i;
-	char udid[41];
-	uint16_t port = 0;
-	udid[0] = 0;
+	char* udid = NULL;
+	lockdownd_service_descriptor_t service = NULL;
 	int cmd = -1;
 	int is_full_backup = 0;
 	char *backup_directory = NULL;
@@ -857,7 +719,7 @@ int main(int argc, char *argv[])
 				print_usage(argc, argv);
 				return 0;
 			}
-			strcpy(udid, argv[i]);
+			udid = strdup(argv[i]);
 			continue;
 		}
 		else if (!strcmp(argv[i], "-h") || !strcmp(argv[i], "--help")) {
@@ -910,8 +772,8 @@ int main(int argc, char *argv[])
 
 	printf("Backup directory is \"%s\"\n", backup_directory);
 
-	if (udid[0] != 0) {
-		ret = idevice_new(&phone, udid);
+	if (udid) {
+		ret = idevice_new(&device, udid);
 		if (ret != IDEVICE_E_SUCCESS) {
 			printf("No device found with udid %s, is it plugged in?\n", udid);
 			return -1;
@@ -919,23 +781,45 @@ int main(int argc, char *argv[])
 	}
 	else
 	{
-		ret = idevice_new(&phone, NULL);
+		ret = idevice_new(&device, NULL);
 		if (ret != IDEVICE_E_SUCCESS) {
 			printf("No device found, is it plugged in?\n");
 			return -1;
 		}
 	}
 
-	if (LOCKDOWN_E_SUCCESS != lockdownd_client_new_with_handshake(phone, &client, "idevicebackup")) {
-		idevice_free(phone);
+	if (LOCKDOWN_E_SUCCESS != (ldret = lockdownd_client_new_with_handshake(device, &client, "idevicebackup"))) {
+		printf("ERROR: Could not connect to lockdownd, error code %d\n", ldret);
+		idevice_free(device);
 		return -1;
 	}
 
+	node = NULL;
+	lockdownd_get_value(client, NULL, "ProductVersion", &node);
+	if (node) {
+		char* str = NULL;
+		if (plist_get_node_type(node) == PLIST_STRING) {
+			plist_get_string_val(node, &str);
+		}
+		plist_free(node);
+		node = NULL;
+		if (str) {
+			int maj = strtol(str, NULL, 10);
+			free(str);
+			if (maj > 3) {
+				printf("ERROR: This tool is only compatible with iOS 3 or below. For newer iOS versions please use the idevicebackup2 tool.\n");
+				lockdownd_client_free(client);
+				idevice_free(device);
+				return -1;
+			}
+		}
+	}
+
 	/* start notification_proxy */
 	np_client_t np = NULL;
-	ret = lockdownd_start_service(client, NP_SERVICE_NAME, &port);
-	if ((ret == LOCKDOWN_E_SUCCESS) && port) {
-		np_client_new(phone, port, &np);
+	ldret = lockdownd_start_service(client, NP_SERVICE_NAME, &service);
+	if ((ldret == LOCKDOWN_E_SUCCESS) && service && service->port) {
+		np_client_new(device, service, &np);
 		np_set_notify_callback(np, notify_cb, NULL);
 		const char *noties[5] = {
 			NP_SYNC_CANCEL_REQUEST,
@@ -952,19 +836,29 @@ int main(int argc, char *argv[])
 	afc_client_t afc = NULL;
 	if (cmd == CMD_BACKUP) {
 		/* start AFC, we need this for the lock file */
-		port = 0;
-		ret = lockdownd_start_service(client, "com.apple.afc", &port);
-		if ((ret == LOCKDOWN_E_SUCCESS) && port) {
-			afc_client_new(phone, port, &afc);
+		service->port = 0;
+		service->ssl_enabled = 0;
+		ldret = lockdownd_start_service(client, "com.apple.afc", &service);
+		if ((ldret == LOCKDOWN_E_SUCCESS) && service->port) {
+			afc_client_new(device, service, &afc);
 		}
 	}
 
+	if (service) {
+		lockdownd_service_descriptor_free(service);
+		service = NULL;
+	}
+
 	/* start mobilebackup service and retrieve port */
-	port = 0;
-	ret = lockdownd_start_service(client, MOBILEBACKUP_SERVICE_NAME, &port);
-	if ((ret == LOCKDOWN_E_SUCCESS) && port) {
-		printf("Started \"%s\" service on port %d.\n", MOBILEBACKUP_SERVICE_NAME, port);
-		mobilebackup_client_new(phone, port, &mobilebackup);
+	ldret = lockdownd_start_service(client, MOBILEBACKUP_SERVICE_NAME, &service);
+	if ((ldret == LOCKDOWN_E_SUCCESS) && service && service->port) {
+		printf("Started \"%s\" service on port %d.\n", MOBILEBACKUP_SERVICE_NAME, service->port);
+		mobilebackup_client_new(device, service, &mobilebackup);
+
+		if (service) {
+			lockdownd_service_descriptor_free(service);
+			service = NULL;
+		}
 
 		/* check abort conditions */
 		if (quit_flag > 0) {
@@ -1046,7 +940,7 @@ int main(int argc, char *argv[])
 			case CMD_BACKUP:
 			printf("Starting backup...\n");
 			/* TODO: check domain com.apple.mobile.backup key RequiresEncrypt and WillEncrypt with lockdown */
-			/* TODO: verify battery on AC enough battery remaining */	
+			/* TODO: verify battery on AC enough battery remaining */
 
 			/* read the last Manifest.plist */
 			if (!is_full_backup) {
@@ -1068,7 +962,7 @@ int main(int argc, char *argv[])
 				}
 				remove(info_path);
 				printf("Creating Info.plist for new backup.\n");
-				info_plist = mobilebackup_factory_info_plist_new();
+				info_plist = mobilebackup_factory_info_plist_new(udid);
 				plist_write_to_filename(info_plist, info_path, PLIST_FORMAT_XML);
 			}
 			free(info_path);
@@ -1132,7 +1026,7 @@ int main(int argc, char *argv[])
 					sleep(2);
 					goto files_out;
 				}
-				
+
 				node = plist_array_get_item(message, 0);
 
 				/* get out if we don't get a DLSendFile */
@@ -1146,7 +1040,7 @@ int main(int argc, char *argv[])
 					node = plist_dict_get_item(node_tmp, "BackupTotalSizeKey");
 					if (node) {
 						plist_get_uint_val(node, &backup_total_size);
-						format_size = format_size_for_display(backup_total_size);
+						format_size = string_format_size(backup_total_size);
 						printf("Backup data requires %s on the disk.\n", format_size);
 						free(format_size);
 					}
@@ -1176,15 +1070,15 @@ int main(int argc, char *argv[])
 					plist_get_uint_val(node, &file_size);
 					backup_real_size += file_size;
 
-					format_size = format_size_for_display(backup_real_size);
+					format_size = string_format_size(backup_real_size);
 					printf("(%s", format_size);
 					free(format_size);
 
-					format_size = format_size_for_display(backup_total_size);
+					format_size = string_format_size(backup_total_size);
 					printf("/%s): ", format_size);
 					free(format_size);
 
-					format_size = format_size_for_display(file_size);
+					format_size = string_format_size(file_size);
 					printf("Receiving file %s (%s)... \n", filename_source, format_size);
 					free(format_size);
 
@@ -1393,7 +1287,7 @@ files_out:
 					}
 					free(auth_sig);
 				} else if (auth_ver) {
-					printf("Unknown AuthVersion '%s', cannot verify AuthSignature\n", auth_ver); 
+					printf("Unknown AuthVersion '%s', cannot verify AuthSignature\n", auth_ver);
 				}
 				plist_from_bin(bin, (uint32_t)binsize, &backup_data);
 				free(bin);
@@ -1496,42 +1390,62 @@ files_out:
 						printf("Restoring file %s %d/%d (%d%%)... ", file_path, cur_file, total_files, (cur_file*100/total_files));
 
 						/* add additional device link file information keys */
-						plist_dict_insert_item(file_info, "DLFileAttributesKey", plist_copy(node));
-						plist_dict_insert_item(file_info, "DLFileSource", plist_new_string(file_info_path));
-						plist_dict_insert_item(file_info, "DLFileDest", plist_new_string("/tmp/RestoreFile.plist"));
-						plist_dict_insert_item(file_info, "DLFileIsEncrypted", plist_new_bool(is_encrypted));
-						plist_dict_insert_item(file_info, "DLFileOffsetKey", plist_new_uint(file_offset));
-						plist_dict_insert_item(file_info, "DLFileStatusKey", plist_new_uint(file_status));
+						plist_dict_set_item(file_info, "DLFileAttributesKey", plist_copy(node));
+						plist_dict_set_item(file_info, "DLFileSource", plist_new_string(file_info_path));
+						plist_dict_set_item(file_info, "DLFileDest", plist_new_string("/tmp/RestoreFile.plist"));
+						plist_dict_set_item(file_info, "DLFileIsEncrypted", plist_new_bool(is_encrypted));
+						plist_dict_set_item(file_info, "DLFileOffsetKey", plist_new_uint(file_offset));
+						plist_dict_set_item(file_info, "DLFileStatusKey", plist_new_uint(file_status));
 
 						/* read data from file */
 						free(file_info_path);
 						file_info_path = mobilebackup_build_path(backup_directory, hash, ".mddata");
-						buffer_read_from_filename(file_info_path, &buffer, &length);
+
+						/* determine file size */
+#ifdef WIN32
+						struct _stati64 fst;
+						if (_stati64(file_info_path, &fst) != 0)
+#else
+						struct stat fst;
+						if (stat(file_info_path, &fst) != 0)
+#endif
+						{
+							printf("ERROR: stat() failed for '%s': %s\n", file_info_path, strerror(errno));
+							free(file_info_path);
+							break;
+						}
+						length = fst.st_size;
+
+						FILE *f = fopen(file_info_path, "rb");
+						if (!f) {
+							printf("ERROR: could not open local file '%s': %s\n", file_info_path, strerror(errno));
+							free(file_info_path);
+							break;
+						}
 						free(file_info_path);
 
 						/* send DLSendFile messages */
 						file_offset = 0;
 						do {
-							if ((length-file_offset) <= 8192)
+							char buf[8192];
+							size_t len = fread(buf, 1, sizeof(buf), f);
+
+							if ((length-file_offset) <= sizeof(buf))
 								file_status = DEVICE_LINK_FILE_STATUS_LAST_HUNK;
 							else
 								file_status = DEVICE_LINK_FILE_STATUS_HUNK;
-							
+
 							plist_dict_remove_item(file_info, "DLFileOffsetKey");
-							plist_dict_insert_item(file_info, "DLFileOffsetKey", plist_new_uint(file_offset));
+							plist_dict_set_item(file_info, "DLFileOffsetKey", plist_new_uint(file_offset));
 
 							plist_dict_remove_item(file_info, "DLFileStatusKey");
-							plist_dict_insert_item(file_info, "DLFileStatusKey", plist_new_uint(file_status));
+							plist_dict_set_item(file_info, "DLFileStatusKey", plist_new_uint(file_status));
 
 							send_file_node = plist_new_array();
 
 							plist_array_append_item(send_file_node, plist_new_string("DLSendFile"));
 
-							if (file_status == DEVICE_LINK_FILE_STATUS_LAST_HUNK)
-								plist_array_append_item(send_file_node, plist_new_data(buffer+file_offset, length-file_offset));
-							else
-								plist_array_append_item(send_file_node, plist_new_data(buffer+file_offset, 8192));
-
+							plist_array_append_item(send_file_node, plist_new_data(buf, len));
 							plist_array_append_item(send_file_node, plist_copy(file_info));
 
 							err = mobilebackup_send(mobilebackup, send_file_node);
@@ -1549,13 +1463,13 @@ files_out:
 								}
 							}
 
-							file_offset += 8192;
+							file_offset += len;
 
 							if (file_status == DEVICE_LINK_FILE_STATUS_LAST_HUNK)
 								printf("DONE\n");
 
 							plist_free(send_file_node);
-							
+
 							if (file_status == DEVICE_LINK_FILE_STATUS_NONE)
 								break;
 
@@ -1602,8 +1516,8 @@ files_out:
 						tmp_node = plist_dict_get_item(node, "AppInfo");
 
 						dict = plist_new_dict();
-						plist_dict_insert_item(dict, "AppInfo", plist_copy(tmp_node));
-						plist_dict_insert_item(dict, "BackupMessageTypeKey", plist_new_string("BackupMessageRestoreApplicationSent"));
+						plist_dict_set_item(dict, "AppInfo", plist_copy(tmp_node));
+						plist_dict_set_item(dict, "BackupMessageTypeKey", plist_new_string("BackupMessageRestoreApplicationSent"));
 
 						array = plist_new_array();
 						plist_array_append_item(array, plist_new_string("DLMessageProcessMessage"));
@@ -1697,7 +1611,11 @@ files_out:
 	if (mobilebackup)
 		mobilebackup_client_free(mobilebackup);
 
-	idevice_free(phone);
+	idevice_free(device);
+
+	if (udid) {
+		free(udid);
+	}
 
 	return 0;
 }
