@@ -8,15 +8,15 @@
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA 
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include <stdio.h>
@@ -30,23 +30,24 @@
 static void print_usage(int argc, char **argv)
 {
 	char *name = NULL;
-	
+
 	name = strrchr(argv[0], '/');
 	printf("Usage: %s [OPTIONS] UDID\n", (name ? name + 1: argv[0]));
 	printf("Makes a device with the supplied 40-digit UDID enter recovery mode immediately.\n\n");
 	printf("  -d, --debug\t\tenable communication debugging\n");
 	printf("  -h, --help\t\tprints usage information\n");
 	printf("\n");
+	printf("Homepage: <http://libimobiledevice.org>\n");
 }
 
 int main(int argc, char *argv[])
 {
 	lockdownd_client_t client = NULL;
-	idevice_t phone = NULL;
+	lockdownd_error_t ldret = LOCKDOWN_E_UNKNOWN_ERROR;
+	idevice_t device = NULL;
 	idevice_error_t ret = IDEVICE_E_UNKNOWN_ERROR;
 	int i;
-	char udid[41];
-	udid[0] = 0;
+	const char* udid = NULL;
 
 	/* parse cmdline args */
 	for (i = 1; i < argc; i++) {
@@ -65,16 +66,17 @@ int main(int argc, char *argv[])
 		print_usage(argc, argv);
 		return 0;
 	}
-	strcpy(udid, argv[i]);
+	udid = argv[i];
 
-	ret = idevice_new(&phone, udid);
+	ret = idevice_new(&device, udid);
 	if (ret != IDEVICE_E_SUCCESS) {
 		printf("No device found with udid %s, is it plugged in?\n", udid);
 		return -1;
 	}
 
-	if (LOCKDOWN_E_SUCCESS != lockdownd_client_new(phone, &client, "ideviceenterrecovery")) {
-		idevice_free(phone);
+	if (LOCKDOWN_E_SUCCESS != (ldret = lockdownd_client_new(device, &client, "ideviceenterrecovery"))) {
+		printf("ERROR: Could not connect to lockdownd, error code %d\n", ldret);
+		idevice_free(device);
 		return -1;
 	}
 
@@ -87,7 +89,7 @@ int main(int argc, char *argv[])
 	printf("Device is successfully switching to recovery mode.\n");
 
 	lockdownd_client_free(client);
-	idevice_free(phone);
+	idevice_free(device);
 
 	return 0;
 }
